#!/usr/bin/perl -T

use strict;
use warnings;

#*******************************************************************************
# Data::Cache::Folder module/class tests
#
# Author:   Jan Mach, jan.mach@cesnet.cz
# Version:  0.1
# Purpose:
#
#*******************************************************************************

use Data::Dumper;
#use Smart::Comments;

use Test::More tests => 17;

# First test if the module correctly compiles
BEGIN {
        use vars qw($TESTED_CLASS);
        $TESTED_CLASS = 'Data::Cache::Folder';

        use_ok('Tester');
        use_ok('Data::Cache');
        use_ok($TESTED_CLASS);

        $ENV{PATH} = '/bin:/usr/bin';
    }

# Debug output explicit ON switch
#Tester::DEBUG_ON();
#Tester::DEBUG_ON(10);

# Debug output explicit OFF switch
#Tester::DEBUG_OFF();

# User should know, which version is beeing tested (just for sure)
### $Data::Cache::Folder::VERSION

### Attempting to create Data::Cache::Folder instance...
my $instance = $TESTED_CLASS->new('./spool/cache');
isa_ok($instance, $TESTED_CLASS);

### Viewing the created instance: $instance
diag("Instance dump 01: " . Dumper($instance)) if Tester::D(2);

my $testa = ['array1', 'test', 'values'];
my $testb = {a => '1', b => '1', c => '3'};
my $result;

is($instance->cache('id1', $testa), '', 'Cache test 01');
is($instance->cache('id2', $testb), '', 'Cache test 02');

my $test1 = $instance->fetch('id1');
my $test2 = $instance->fetch('id2');

is_deeply($test1, $testa, 'Fetch test 01');
is_deeply($test2, $testb, 'Fetch test 02');

is($instance->clear(0), 'Deleted 0 cache files older than 0.0000s', 'Clear test 01');

my $ts1 = (stat('./spool/cache/id1.bin'))[9];
my $ts2 = (stat('./spool/cache/id2.bin'))[9];

$test1 = $instance->fetch_newer('id1', $ts1);
$test2 = $instance->fetch_newer('id2', $ts2);

is_deeply($test1, $testa, 'Fetch newer test 01');
is_deeply($test2, $testb, 'Fetch newer test 02');

$test1 = $instance->fetch_newer('id1', $ts1 - 10);
$test2 = $instance->fetch_newer('id2', $ts2 - 10);

is_deeply($test1, $testa, 'Fetch newer test 03');
is_deeply($test2, $testb, 'Fetch newer test 04');

$test1 = $instance->fetch_newer('id1', $ts1 + 10);
$test2 = $instance->fetch_newer('id2', $ts2 + 10);

is($test1, undef, 'Fetch newer test 05');
is($test2, undef, 'Fetch newer test 06');

is(unlink('./spool/cache/id1.bin'), 0, 'Empty cache test 01');
is(unlink('./spool/cache/id2.bin'), 0, 'Empty cache test 02');
