#!/usr/bin/perl -T

use strict;
use warnings;

#*******************************************************************************
# Mentat::Message::Storage::Mongo class tests
#
# Author:   Jan Mach, jan.mach@cesnet.cz
# Version:  0.1
# Purpose:
# Comments:
#*******************************************************************************

use Data::Dumper;
#use Smart::Comments;

use Test::More tests => 4380;

# First test if the module correctly compiles
BEGIN {
        use vars qw($TESTED_CLASS);
        $TESTED_CLASS = 'Mentat::Message::Storage::Mongo';

        use_ok('Tester');
        use_ok('Mentat::Handyman');
        use_ok('Mentat::Storage::Mongo');
        use_ok('Mentat::Message::Factory');
        use_ok('Value::Convertor');
        use_ok('Mentat::Message::Convertor::Mongo');
        use_ok('Mentat::Message::Convertor::IDEA');
        use_ok('Mentat::Message::Builder::IDEA');
        use_ok($TESTED_CLASS);
    }

# Debug output explicit ON switch
#Tester::DEBUG_ON();

# Debug output explicit OFF switch
#Tester::DEBUG_OFF();

# User should know, which version is beeing tested (just for sure)
#### $Mentat::Message::Storage::Mongo::VERSION

#-------------------------------------------------------------------------------

my $CNT = 0;
sub gen_msg_rules()
{
    ++$CNT;
    my $message_rules = [
        ['EventTime','%ts:$detect_time'],
        ['ConnCount','$5'],
        ['Node[1]/SW','Kippo'],
        ['Node[1]/Tags[1]','Network'],
        ['Source[1]/%ipt:$1[1]','$1'],
        ['Target[1]/%ipt:$2[1]','$2'],
        ['Target[1]/Port[1]','22'],
        ['Category[1]','Intrusion'],
        ['Description','User $3 $4 login from $1'],
    ];

    return $message_rules;
}

#-------------------------------------------------------------------------------

# Prepare builder configurations
my $BUILDER_CONFIG = {
        'name' => 'tld.domain.node',
        'type' => 'kippo,honeypot',
        'sw'   => 'kippo',
        'note' => 'Some note',
    };

# Prepare message configurations
my $message_config = {
        'detect_time' => time,
        '1' => '127.0.0.1',
        '2' => '127.0.0.2',
        '3' => 'root',
        '4' => 'failed',
        '5' => 10,
    };

# Now attempt to create a class instance and check it
my $builder = Mentat::Message::Builder::IDEA->new($BUILDER_CONFIG);
isa_ok($builder, 'Mentat::Message::Builder::IDEA');

# Now attempt to create a class instance and check it
my $instance = $TESTED_CLASS->new(database => 'unittest', collection => 'pyrl_alerts');
isa_ok($instance, $TESTED_CLASS);

my ($tcount, $message, $result, $count);
eval {
    $instance->create_indexes();

    # Count current number of records in database
    $tcount = $instance->count();
    #### Number of records currently in database: $tcount

    # Check storing to database
    $message = $builder->build(rules     => gen_msg_rules(),
                               name_args => $message_config);
    isa_ok($message, 'Mentat::Message::IDEA');
    my $record_id1 = $instance->insert_msg($message);
    isnt($record_id1, undef, 'Insert test 01');

    $message = $builder->build(rules     => gen_msg_rules(),
                               name_args => $message_config);
    isa_ok($message, 'Mentat::Message::IDEA');
    my $record_id2 = $instance->insert_msg($message);
    isnt($record_id2, undef, 'Insert test 02');

    $message = $builder->build(rules     => gen_msg_rules(),
                               name_args => $message_config);
    isa_ok($message, 'Mentat::Message::IDEA');
    my $record_id3 = $instance->insert_msg($message);
    isnt($record_id3, undef, 'Insert test 03');

    $message = $builder->build(rules     => gen_msg_rules(),
                               name_args => $message_config);
    isa_ok($message, 'Mentat::Message::IDEA');
    my $record_id4 = $instance->insert_msg($message);
    isnt($record_id4, undef, 'Insert test 04');

    $message = $builder->build(rules     => gen_msg_rules(),
                               name_args => $message_config);
    isa_ok($message, 'Mentat::Message::IDEA');
    my $record_id5 = $instance->insert_msg($message);
    isnt($record_id5, undef, 'Insert test 05');
};
if($@) {
    print STDERR "MongoDB error: $@\n";
    $instance->remove();
}
